package org.openntf.sourcesniffer.jobs;

/*
 * (C)Copyright Ferry Kranenburg 2010
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at:
 * 
 * http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * permissions and limitations under the License.
 */

import org.openntf.sourcesniffer.*;
import org.openntf.sourcesniffer.views.ssMainWindow;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;

import org.w3c.dom.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import lotus.domino.Database;
import lotus.domino.DxlExporter;
import lotus.domino.NoteCollection;
import lotus.domino.NotesException;
import lotus.domino.NotesThread;
import lotus.domino.Session;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;


import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.ibm.notes.java.api.util.NotesSessionJob;


public class ssProcessAllFunctionsJob extends NotesSessionJob {

	Database db_Chosendatabase;
	NoteCollection o_DXLNoteCollection;
	DxlExporter o_DXLExporter;
	Session session;
	String str_SelectedServer;
	String str_xml_DatabaseSelectedFile;
	Node xml_ToMemory_Database;
	String sDesignName;
	String str_ElementName;
	Document xmlDoc;
	Document dxlDoc;
	lotus.domino.Document doc_DesignElement;
	Element xml_ToMemory_ElementName;

	



	public ssProcessAllFunctionsJob(Database db_Chosendatabase, Element xml_ToMemory_Database, Document xmldoc) {  
		super("Sniffing DXL design elements..." );
		this.db_Chosendatabase = db_Chosendatabase;
		this.xml_ToMemory_Database = xml_ToMemory_Database; 
		this.str_ElementName = "";
		this.xmlDoc = xmldoc;

	}


	protected IStatus runInNotesThread(Session session, IProgressMonitor iprogressmonitor)	throws NotesException {
		o_DXLNoteCollection = db_Chosendatabase.createNoteCollection(false);
		o_DXLNoteCollection.setSelectDocuments(false);
				
		
		ArrayList<String> srch;

		
		////////////////////////////////////////////////////
		// ACTIONS
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectionFormula("");
		o_DXLNoteCollection.setSelectActions(true);
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("code");
		ProcessDXLDesignElement("Actions", session, iprogressmonitor,  "icons/action20.png", srch);
		
		////////////////////////////////////////////////////
		// AGENTS
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectionFormula("");
		o_DXLNoteCollection.setSelectAgents(true);
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("code");
		ProcessDXLDesignElement("Agents", session, iprogressmonitor, "icons/agent20.png", srch);


		////////////////////////////////////////////////////
		// CUSTOM CONTROLS
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectMiscFormatElements(true);    // 1.1 xpage custom controls weren't recognized properly 
		o_DXLNoteCollection.setSelectionFormula("@Contains($Flags; \"g\") & @Contains($Flags; \";\")") ;
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("$filedata");   // this field has to be base 64 decoded
		ProcessDXLDesignElement("Custom Controls", session, iprogressmonitor, "icons/custom22.png", srch);


		////////////////////////////////////////////////////
		// DATABASESCRIPT
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectionFormula("");
		o_DXLNoteCollection.setSelectDatabaseScript(true);
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("code");
		ProcessDXLDesignElement("Database script", session, iprogressmonitor,  "icons/scriptdatabase20.png", srch);


		////////////////////////////////////////////////////
		// FOLDERS
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectionFormula("");
		o_DXLNoteCollection.setSelectFolders(true);
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("code");
		ProcessDXLDesignElement("Folders", session, iprogressmonitor,  "icons/folder20.png", srch);


		////////////////////////////////////////////////////
		// FORMS
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectionFormula("");
		o_DXLNoteCollection.setSelectForms(true);
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("code");
		srch.add("keywords");
		ProcessDXLDesignElement("Forms", session, iprogressmonitor, "icons/form20.png", srch);
		
		////////////////////////////////////////////////////
		// NAVIGATORS
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectMiscIndexElements(true); 
		o_DXLNoteCollection.setSelectionFormula("@Matches($Flags; \"*G*\")");
		o_DXLNoteCollection.setSelectNavigators(true);  // bug in Lotus Notes 8.5.1, views that are hidden from notes client didn't export! (http://www-10.lotus.com/ldd/bpmpblog.nsf/dx/notesnotecollection.selectviews-nasty-surprise)
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("$viewmaplayout");
		ProcessDXLDesignElement("Navigators", session, iprogressmonitor, "icons/navigator20.png", srch);
		
		
		////////////////////////////////////////////////////
		// OUTLINES
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectionFormula("");
		o_DXLNoteCollection.setSelectOutlines(true);
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("code");
		srch.add("keywords");
		ProcessDXLDesignElement("Outlines", session, iprogressmonitor,  "icons/outline20.png", srch);

		////////////////////////////////////////////////////
		// PAGES
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectionFormula("");
		o_DXLNoteCollection.setSelectPages(true);
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("code");
		ProcessDXLDesignElement("Pages", session, iprogressmonitor,  "icons/page20.png", srch);
		
		////////////////////////////////////////////////////
		// SCRIPT LIBRARIES
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectScriptLibraries(true);
		o_DXLNoteCollection.setSelectionFormula("!@Contains($FlagsExt; \"W\")");   // 1.1 - select only script libraries...
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("code");
		srch.add("item");  // java libraries - decode rawitems
		ProcessDXLDesignElement("Script libraries", session, iprogressmonitor,  "icons/scriptlibrary20.png", srch);
		
		
		////////////////////////////////////////////////////
		// SHARED COLUMNS
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectMiscIndexElements(true);
		o_DXLNoteCollection.setSelectionFormula("@Contains($Flags; \"^\")");
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("code");
		ProcessDXLDesignElement("Shared columns", session, iprogressmonitor, "icons/columns22.png", srch);
		
		////////////////////////////////////////////////////
		// SHARED FIELDS
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectionFormula("");
		o_DXLNoteCollection.setSelectSharedFields(true);
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("code");
		srch.add("keywords");
		ProcessDXLDesignElement("Shared fields", session, iprogressmonitor, "icons/sharedfield20.png", srch);
		
		////////////////////////////////////////////////////
		// SUBFORMS
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectionFormula("");
		o_DXLNoteCollection.setSelectSubforms(true);
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("code");
		srch.add("keywords");
		ProcessDXLDesignElement("Subforms", session, iprogressmonitor,  "icons/subform20.png", srch);
		
		////////////////////////////////////////////////////
		// VIEWS
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectMiscIndexElements(true); 
		o_DXLNoteCollection.setSelectionFormula("!@Matches($Flags; \"*{FG^}*\")");
		o_DXLNoteCollection.setSelectViews(true);  // bug in Lotus Notes 8.5.1, views that are hidden from notes client didn't export! (http://www-10.lotus.com/ldd/bpmpblog.nsf/dx/notesnotecollection.selectviews-nasty-surprise)
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("code");
		ProcessDXLDesignElement("Views", session, iprogressmonitor,  "icons/view20.png", srch);
		
		////////////////////////////////////////////////////
		// WEBSERVICE CONSUME
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectScriptLibraries (true);
		o_DXLNoteCollection.setSelectionFormula("@Contains($FlagsExt; \"W\")");
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("$scriptlib");	
		ProcessDXLDesignElement("Webservice consumer", session, iprogressmonitor,  "icons/webservice_down20.png" ,srch);
		
		////////////////////////////////////////////////////
		// WEBSERVICE PROVIDER
		///////////////////////////////////////////////////
		o_DXLNoteCollection.selectAllDesignElements(false);
		o_DXLNoteCollection.setSelectScriptLibraries (true);
		o_DXLNoteCollection.setSelectionFormula("@Contains($Flags; \"{\")");
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("$scriptlib");	
		ProcessDXLDesignElement("Webservice provider", session, iprogressmonitor,  "icons/webservice_up20.png", srch);

		////////////////////////////////////////////////////
		// XPAGES
		///////////////////////////////////////////////////
		o_DXLNoteCollection.setSelectMiscFormatElements(true);
		o_DXLNoteCollection.setSelectionFormula("@Contains($Flags; \"g\") & @Contains($Flags; \"K\")");
		o_DXLNoteCollection.buildCollection();
		srch = new ArrayList<String>();
		srch.add("$filedata"); // this field has to be base 64 decoded
		ProcessDXLDesignElement("Xpages", session, iprogressmonitor,  "icons/xpage20.png", srch );


		
		return Status.OK_STATUS;

		
	}

	private void ProcessDXLDesignElement(String sDesignName ,Session session, IProgressMonitor iprogressmonitor, String sImagePath, ArrayList<String> arr_CodeNodes) {
		// variabels
		Integer i_DesignElementCount = 0	;
		doc_DesignElement = null;
		Boolean b_SkipBecauseOfActions = false ; 
		// get noteID
		String str_NoteID = "";

		// create domino DXLexporter object
		try {
			// if monitor had been canceled then return!
			if (iprogressmonitor.isCanceled() == true) {
				// reset str_Server and str_Database vars...
				ssMainWindow.str_SelectedFile = "";
				ssMainWindow.str_SelectedServer = "";
				iprogressmonitor.done();
				return;

			}
			
			// initialize lotus notes 
			NotesThread.sinitThread();

			o_DXLExporter = session.createDxlExporter();
			o_DXLExporter.setOutputDOCTYPE(false);   //'if true then we need dtd schema's..
			o_DXLExporter.setForceNoteFormat(false);
		
			

			// update progress monitor
			iprogressmonitor.beginTask("Sniffing " + sDesignName + "....",o_DXLNoteCollection.getCount() );
			

			// create design element node (all in-memory, so we can populate the gui treeviewer later)
			Element xml_ToMemory_DBelement = null;
			xml_ToMemory_DBelement = xmlDoc.createElement("DesignName");
			xml_ToMemory_DBelement.setAttribute("ss_Name", sDesignName);
			xml_ToMemory_DBelement.setAttribute("ss_Image", sImagePath);
			xml_ToMemory_Database.appendChild(xml_ToMemory_DBelement);



			str_NoteID = o_DXLNoteCollection.getFirstNoteID();

			while (str_NoteID != null && str_NoteID.length() > 0) {
				i_DesignElementCount++;
				doc_DesignElement = db_Chosendatabase.getDocumentByID(str_NoteID);


				if(doc_DesignElement != null){
					String str_XML = null;

					
					
					str_XML = o_DXLExporter.exportDxl(doc_DesignElement);
					str_XML = str_XML.replace("<?xml version='1.0'?>", "<?xml version='1.0' encoding='ISO-8859-1'?>");


					if(str_XML!= "") {
						//'load the xml file
						DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
						factory.setNamespaceAware(true); 
						DocumentBuilder db = factory.newDocumentBuilder();
						InputSource is = new InputSource();
						is.setCharacterStream(new StringReader(str_XML));

						dxlDoc = db.parse(is);
						   dxlDoc.getDocumentElement().normalize();



						// title of scriptelement, strip aliases (because of the show references possibility)
						str_ElementName = doc_DesignElement.getItemValueString("$TITLE");


						if (str_ElementName.indexOf("|") > 0) {
							str_ElementName =  ssTools.strMiddleOfString(str_ElementName.toString(), "", "|", false, false, false);
						}



						iprogressmonitor.setTaskName("sniffing " + sDesignName + " (" + i_DesignElementCount + "/" + o_DXLNoteCollection.getCount() + ") - " + str_ElementName);
						iprogressmonitor.worked(1);

						xml_ToMemory_ElementName = null;
						xml_ToMemory_ElementName = xmlDoc.createElement("Element");
						xml_ToMemory_ElementName.setAttribute("ss_Name", str_ElementName);
						xml_ToMemory_ElementName.setAttribute("ss_Image", sImagePath);
						


						// aanroep get dxl elements
						
						ProcessAllDxlCodeElements(dxlDoc.getDocumentElement(), xml_ToMemory_ElementName, sImagePath, true,arr_CodeNodes );
				
						

						xml_ToMemory_Database.getLastChild().appendChild(xml_ToMemory_ElementName);
						

					}
				}  //if

				//for actions we must loop one time, else go to next note
				if( b_SkipBecauseOfActions == true) { 
					str_NoteID = "";
				}
				else{
					str_NoteID = o_DXLNoteCollection.getNextNoteID(str_NoteID);
				}
			}; //while en do

	
		} catch (NotesException e) {
			e.printStackTrace();

		} catch (ParserConfigurationException e) {
				e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			NotesThread.stermThread();
		}


	}
	
	
	private void ProcessAllDxlCodeElements(Node DXL_RootNode, Element xml_ToMemory_ElementName, String sImagePath, boolean bProcessRootNode, ArrayList<String> arr_CodeNodes  ) throws Exception, NotesException{

		if( DXL_RootNode.hasChildNodes() == true) {
			NodeList nlDXLprocesnodes = DXL_RootNode.getChildNodes() ;
			String str_Prefix;
			String str_DxlAttributes = "";
			String str_ActionName;
			String str_NameOfDesignElement;
			Element xml_ToMemory_EventCodeNode = null;

			//////////////////////////
			// BEGIN CHILD PROCESSING
			//////////////////////////	
			for(Integer iNode =  0 ;iNode < nlDXLprocesnodes.getLength(); iNode++){
				Node DXL_ChildNode = nlDXLprocesnodes.item(iNode);
				if (DXL_ChildNode.getNodeType() == Node.ELEMENT_NODE ) {

					//////////////////////////
					// GET ATTRIBUTES
					//////////////////////////
					str_DxlAttributes =  ssTools.getAllAttributesToString(DXL_ChildNode);


					//////////////////////////////////
					// TRY TO DESIGN ELEMENT SUB NAME
					//////////////////////////////////
					str_ActionName = DXL_ChildNode.getNodeName().toString().toLowerCase();


					if (str_ActionName == "action") {
						str_NameOfDesignElement = ssTools.getAttributeByName(DXL_ChildNode, "title", false, false);
						str_Prefix = " (action) ";
					}
					else if ( str_ActionName == "actionhotspot"){
						if (DXL_ChildNode.hasChildNodes() == true ){
							str_NameOfDesignElement = ssTools.strMiddleOfString(DXL_ChildNode.getFirstChild().getTextContent(), "", "Sub", false, true, false);
						}else{
							str_NameOfDesignElement = ssTools.strMiddleOfString(DXL_ChildNode.getTextContent(), "", "Sub", false, true, false);
						}
						str_Prefix = " (action hotspot) ";
					}
					else if ( str_ActionName == "button"){
						str_NameOfDesignElement = ssTools.getAttributeByName(DXL_ChildNode, "name", false, false);
						if (str_NameOfDesignElement.equals(null) || str_NameOfDesignElement.equals("")){
							//v1.6 - replace any formula code in the child elements, then the result is likely the button name...
							str_NameOfDesignElement = DXL_ChildNode.getTextContent();
											
							NodeList nlChildren = DXL_ChildNode.getChildNodes();
							for(Integer iChld =  0 ;iChld < nlChildren.getLength(); iChld++){
								Node nCurrentChild = DXL_ChildNode.getChildNodes().item(iChld);
							    if (nCurrentChild.getNodeType() == Node.ELEMENT_NODE){
								   str_NameOfDesignElement = str_NameOfDesignElement.replace(nCurrentChild.getTextContent(), "");
								 }
							}
						}
						if (str_NameOfDesignElement.equals(null) || str_NameOfDesignElement.equals("")){
							str_NameOfDesignElement = DXL_ChildNode.getTextContent().split("\n")[0];
						}						
						str_Prefix = " (button) ";
					}
					else if ( str_ActionName == "column"){
						str_NameOfDesignElement =  ssTools.getAttributeByName(DXL_ChildNode, "itemname", false, false);
						str_Prefix = " (column) ";
					}
					else if ( str_ActionName == "embeddedview"){
						str_NameOfDesignElement =  ssTools.getAttributeByName(DXL_ChildNode, "name", false, false);
						str_Prefix = " (embedded view) ";
					}
					else if (str_ActionName == "field"){
						str_NameOfDesignElement =  ssTools.getAttributeByName(DXL_ChildNode, "name", false, false);
						str_Prefix = " (field) ";

					}
					else if (str_ActionName == "globals"){
						str_NameOfDesignElement = "Globals";
						str_Prefix = "";
					}
					else if (str_ActionName == "sharedactionref"){
						str_NameOfDesignElement =  ssTools.getAttributeByName(DXL_ChildNode.getFirstChild(), "title", false, false);
						str_Prefix = " (shared action) ";
					}
					else{
						str_NameOfDesignElement = "" ;
						str_Prefix = "Locals ";
					}

					////////////////////////////////////////////////////////////////////////////////////////////////////
					// create in-memory xml eventcode node, vars prefix and name of element would be filled above
					////////////////////////////////////////////////////////////////////////////////////////////////////
					xml_ToMemory_EventCodeNode = xmlDoc.createElement("Element");
					xml_ToMemory_EventCodeNode.setAttribute("ss_Name", str_Prefix + str_NameOfDesignElement);
					xml_ToMemory_EventCodeNode.setAttribute("ss_Image", sImagePath);
					xml_ToMemory_EventCodeNode.setAttribute("ss_NotesUrl", doc_DesignElement.getNotesURL().toString());
					xml_ToMemory_EventCodeNode.setAttribute("ss_Attributes", str_DxlAttributes);


					////////////////////////////////////////////////////////////////////////////////////////////////////
					//ADD ALL CODE NODES
					////////////////////////////////////////////////////////////////////////////////////////////////////

					
					ssTools.addAllNodesByName(DXL_ChildNode,arr_CodeNodes,xml_ToMemory_EventCodeNode, xmlDoc, sImagePath,doc_DesignElement.getNotesURL().toString());
					if (bProcessRootNode == true){
						
						ssTools.addAllNodesByName(DXL_ChildNode.getParentNode(),arr_CodeNodes,xml_ToMemory_EventCodeNode, xmlDoc, sImagePath,doc_DesignElement.getNotesURL().toString());
						bProcessRootNode = false;   // DO THIS FIRST TIME ROUND ONLY!
					}
		
					 
					////////////////////////////////////////////////////////////////////////////////////////////////////
					//IF EVENT NODE HAD CHILDREN THEN ADD IT
					////////////////////////////////////////////////////////////////////////////////////////////////////
					if (xml_ToMemory_EventCodeNode.getChildNodes().getLength() > 0) {
						xml_ToMemory_ElementName.appendChild(xml_ToMemory_EventCodeNode);
					}

					//////////////////////////
					// RECURSIVE CALL TO THIS FUNCTION
					//////////////////////////
					ProcessAllDxlCodeElements(DXL_ChildNode, xml_ToMemory_ElementName, sImagePath, bProcessRootNode, arr_CodeNodes );
				}//end if elementNode

			}// next childnode

		}

	}// end ProcessAllDxlElements

	


}//end class



