package org.openntf.sourcesniffer.views;

/*
 * (C)Copyright Ferry Kranenburg 2010
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at:
 * 
 * http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * permissions and limitations under the License.
 */

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.graphics.Region;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Canvas;

import org.openntf.sourcesniffer.Activator;
import org.openntf.sourcesniffer.ssTools;
import org.openntf.sourcesniffer.views.ssMainWindow;

import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;

public class ssFindDialog extends Dialog {
	private Text text;
	private Display display;
	private int i_Searchfrom = 0;
	private Button btnFind;

	public ssFindDialog(Shell shell, Display display) {
		super(shell, SWT.DIALOG_TRIM | SWT.MODELESS);
		this.display = display;

	}

	public void open() {
		final Shell shell = new Shell(display, SWT.DIALOG_TRIM);
		shell.setBackground(new Color(null, 255, 255, 255));
		shell.setMinimumSize(new Point(400, 150));
		shell.setSize(418, 150);
		shell.setText("Find text");
		shell.pack();
		Region region = new Region();

		int rec_width = 418;
		int rec_height = 150;

		Rectangle pixel = new Rectangle(0, 0, rec_width, rec_height); 
		region.add(pixel); 
		shell.setRegion(region);

		int height = display.getBounds().height - rec_height; 
		int width = display.getBounds().width - rec_width; 
		shell.setLocation(width/2,height/2); 


		shell.setLayout(new GridLayout(3, false));

		Canvas canvas = new Canvas(shell, SWT.NONE);
		GridData gd_canvas = new GridData(SWT.LEFT, SWT.TOP, false, true, 1, 2);
		gd_canvas.heightHint = 106;
		gd_canvas.widthHint = 101;
		canvas.setLayoutData(gd_canvas);
		canvas.setToolTipText("developed by Ferry Kranenburg");
		canvas.setBackgroundImage(Activator.getImageDescriptor("icons/logo-source-sniffer-DEF.jpg").createImage());

		text = new Text(shell, SWT.BORDER);
		text.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent key) {
				if (key.keyCode == 13) {
					// on enter, set focus & select the find button
					btnFind.setFocus();
					if(! text.getText().equals("")){
						i_Searchfrom = ssTools.scrollToFirstText(ssMainWindow.richText,  text.getText(), i_Searchfrom);
					}
				}
			}
		});
		GridData gd_text = new GridData(SWT.CENTER, SWT.CENTER, false, true, 1, 1);
		gd_text.widthHint = 179;
		text.setLayoutData(gd_text);

		btnFind = new Button(shell, SWT.NONE);
		btnFind.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				if(! text.getText().equals("")){
					i_Searchfrom = ssTools.scrollToFirstText(ssMainWindow.richText,  text.getText(), i_Searchfrom);
				}
			}
		});
		btnFind.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true, 1, 1));
		btnFind.setText("Find (next)");
		new Label(shell, SWT.NONE);

		Button btnClose = new Button(shell, SWT.NONE);
		btnClose.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true, 1, 1));
		btnClose.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				shell.close();
			}
		});
		btnClose.setText("Close");

		shell.open();
		display = getParent().getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

	}
}

