package org.openntf.sourcesniffer.views;

/*
 * (C)Copyright Ferry Kranenburg 2010
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at:
 * 
 * http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * permissions and limitations under the License.
 */

import java.io.FileNotFoundException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;


import lotus.domino.Database;
import lotus.domino.NotesException;
import lotus.domino.NotesFactory;
import lotus.domino.NotesThread;
import lotus.domino.Session;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.openntf.sourcesniffer.Activator;
import org.openntf.sourcesniffer.ssTools;
import org.openntf.sourcesniffer.jobs.ssProcessAllFunctionsJob;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.ibm.designer.domino.ui.commons.extensions.DesignerDesignElementSelection;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

public class ssMainWindow extends ViewPart {

	public static final String ID = "org.openntf.sourcesniffer.views.ssMainWindow"; //$NON-NLS-1$
	public static String str_SelectedServer = "";
	public static String str_SelectedFile = "" ;
	public static String str_Filter = "" ;


	private Session session;
	public static Action actionInputCurrentElement;
	private Database db_Chosendatabase;
	private String sDatabaseTitle;

	private StyledText richText;	
	public static TreeViewer treeViewer;
	private Canvas canvas;
	public static Text textFilter;

	
	// currently selected designer element
	public static DesignerDesignElementSelection designerSelection = null;

	
	private Document xml_Inmemory_Doc = null;
	private DocumentBuilderFactory xml_DocFactory;
	private DocumentBuilder xml_DocBuilder;
	private DOMImplementation xml_DOMimpl; 
	// create database node
	private Element  xml_Database = null;

	
	
	private Tree tree;
	private Button btnX;

	private TabFolder tabFolder;
	private TabItem tbtmSourceCode;
	private TabItem tbtmAttributes;
	private Table table;
	private TableColumn tblclmnAttributeName;
	private TableColumn tblclmnAttributeValue;
	public static Label lblDatabase;
	private TabItem tbtmUnusedVariables;
	public static Table tblUnusedVariables;
	private TableColumn tblclmnVariableName;
	private TableColumn tblclmnCount;
	private TabItem tbtmStatistics;
	public static Table tblStatistics;
	private TableColumn tblclmnData;
	private TableColumn tblclmnAnalysis;
	
	

	public ssMainWindow() {
		setTitleImage(Activator.getImageDescriptor("icons/logo32.png").createImage());
		setPartName("Source Sniffer v1.2");
		// initialize lotus notes 
		NotesThread.sinitThread();
		try {
			session = NotesFactory.createSession() ;
			str_SelectedServer = "";
			str_SelectedFile = "" ;
			str_Filter = "" ;

			
		} 
		catch (NotesException e) {
			e.printStackTrace();
		} finally {
		     NotesThread.stermThread();
		}

	}

	@Override
	public void createPartControl(Composite parent) {
		parent.setBackground(new Color(null, 255, 255, 255));
		GridLayout gl_parent = new GridLayout(5, false);
		gl_parent.horizontalSpacing = 10;
		gl_parent.verticalSpacing = 10;
		gl_parent.marginBottom = 1;
		gl_parent.marginHeight = 10;

		parent.getShell();
		parent.setLayout(gl_parent);
	    
	    canvas = new Canvas(parent, SWT.NONE);
	    canvas.setToolTipText("developed by Ferry Kranenburg");
	    GridData gd_canvas = new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 2);
	    gd_canvas.heightHint = 106;
	    gd_canvas.widthHint = 100;
	    canvas.setBackgroundImage(Activator.getImageDescriptor("icons/logo-source-sniffer-DEF.jpg").createImage());
	    canvas.setLayoutData(gd_canvas);
	    
	    		
	    textFilter = new Text(parent, SWT.BORDER);
	    textFilter.setToolTipText("type filter text here and press enter");
	    textFilter.addKeyListener(new KeyAdapter() {
	    	public void keyPressed(KeyEvent key) {
    	        if (key.keyCode == 13) {
	            	treeViewer.refresh();   // on enter, refresh treeview
	            }
	    	}
	    });
	    textFilter.addModifyListener(new ModifyListener() {
	    	public void modifyText(ModifyEvent event) {
	    		str_Filter = textFilter.getText();
	    		if (str_Filter.equals("")){
	    			treeViewer.refresh();   // on 'nothing', refresh treeview
	    			btnX.setVisible(false);
	    		}else{
	    			btnX.setVisible(true);
	    		}
	    	}
	    });
   	

	    
	    textFilter.setFont(new Font(null, "Segoe UI", 9, SWT.BOLD));
	    textFilter.setBackground (new Color(null, 240, 240, 240));
	    textFilter.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
	    
	    // add 'text filter remove' button
	    btnX = new Button(parent, SWT.NONE);
	    btnX.addSelectionListener(new SelectionAdapter() {
	    	public void widgetSelected(SelectionEvent buttonEvent) {
	    		textFilter.setText("");
	    		treeViewer.refresh();   // on 'nothing', refresh treeview
	    		btnX.setVisible(false);
	    	}
	    });
	    btnX.setToolTipText("remove filter");
	    btnX.setForeground(new Color(null, 255, 69, 0));
	    btnX.setImage(Activator.getImageDescriptor("icons/cross16.png").createImage());
	    
	    lblDatabase = new Label(parent, SWT.NONE);
	    lblDatabase.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
	    lblDatabase.setText("   ");


	    // create the tree itself
	    tree = new Tree(parent, SWT.BORDER);
	    tree.setFont(new Font(null, "Arial", 10, SWT.NORMAL));
	    tree.setForeground(new Color(null, 0, 0, 0));
	    tree.setBackground(new Color(null, 245, 245, 245));
	    tree.setLinesVisible(true);
	    tree.setSortDirection(SWT.DOWN);
	    GridData gd_tree = new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1);
	    gd_tree.widthHint = 352;
	    gd_tree.heightHint = 175;
	    tree.setLayoutData(gd_tree);

	    // create a treeviewer
	    treeViewer = new TreeViewer(tree);
		// set treeviewer how to visualize the treecontent
		treeViewer.setLabelProvider(new ssTreeLabelProvider());
		// set treeview how to return the treecontent
		treeViewer.setContentProvider( new ssTreeContentProvider());


		// when treeviewer selection changes in treeviewer, display sourcecode ans attributes
		treeViewer.addSelectionChangedListener(new ISelectionChangedListener(){
			public void selectionChanged(SelectionChangedEvent event) {
				if(event.getSelection().isEmpty()) {
					return;
				}
				TreeItem[] clickedTreeItem = treeViewer.getTree().getSelection();
				Node hoveredNode  = (Node) clickedTreeItem[0].getData();
				String str_SourceCode = ssTools.getAttributeByName(hoveredNode, "ss_SourceCode", false, true );	
				String str_Attributes = ssTools.getAttributeByName(hoveredNode, "ss_Attributes", false, true );	

				// update statistics
				tblStatistics.removeAll();
				TableItem tblStatItem;
				
				String tmpString = ssTools.getAttributeByName(hoveredNode, "ss_SourceCodeLineCount", false, false ).trim();
				if (tmpString.equals("") == false ){
					tblStatItem = new TableItem(tblStatistics, SWT.NONE);
					tblStatItem.setText(0, "Current count code lines");
					tblStatItem.setText(1,  tmpString);
				}else{
					tblStatItem = new TableItem(tblStatistics, SWT.NONE);
					tblStatItem.setText(0, "Total count code lines");
					tblStatItem.setText(1,  String.valueOf(ssTools.getSourceCodeLineCount(hoveredNode,0, false)));
				}
				tblStatItem = new TableItem(tblStatistics, SWT.NONE);
				tblStatItem.setText(0, "Nested design elements");
				tblStatItem.setText(1, String.valueOf(hoveredNode.getChildNodes().getLength() ));
				
				
				
				table.removeAll();
				
				if (str_Attributes.equals("") == false) {
				    String strAtts[] =   str_Attributes.split("\r\n");
				    for (String sAtt : strAtts) {
				    	if (sAtt.equals("") == false) {
					    	TableItem tblItem = new TableItem(table, SWT.NONE);
					    	String subStr[] = sAtt.split(",");
					    	
					    	if (subStr.length == 2) {
						    	tblItem.setText(0, subStr[0]);
						    	tblItem.setText(1, subStr[1]);
					    	}
				    	}
				    }
				}
				
				// add all source code to the styled text box
				richText.setText(str_SourceCode);

				// v1.2 - get unused variables
				ssTools.getUnusedVariablesInCode(richText);
				
				// highlight on the current filter
				if (!str_SourceCode.equals("")){
					if (!ssMainWindow.str_Filter.equals("")){
						// highlight alle words that equals the filter text
						ssTools.highLightWord(richText,ssMainWindow.str_Filter, treeViewer.getTree().getDisplay().getSystemColor (SWT.COLOR_GREEN)  );
					}
				}
			}
		});



		// create static action to get the current design element selection
		actionInputCurrentElement = new Action() {
			public void run() {
				if(!(designerSelection == null)){
					if (designerSelection instanceof DesignerDesignElementSelection){
						if (str_SelectedFile.toLowerCase().equals(designerSelection.getSelectedDesignerProject().getDatabaseName().toLowerCase()) == false){
							str_SelectedServer = designerSelection.getSelectedDesignerProject().getServerName();
							str_SelectedFile = designerSelection.getSelectedDesignerProject().getDatabaseName();


							// initialize the source code sniffing!
							ssInitScriptSniffing();

						}

					}

				}
			}
		};

		tabFolder = new TabFolder(parent, SWT.NONE);
		tabFolder.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
		
		tbtmSourceCode = new TabItem(tabFolder, SWT.NONE);
		tbtmSourceCode.setText("Source code");
		

		// create styled text for the source code preview
		richText = new StyledText(tabFolder,  SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		tbtmSourceCode.setControl(richText);
		richText.setForeground(treeViewer.getTree().getDisplay().getSystemColor (SWT.COLOR_BLACK));
		richText.setBackground (treeViewer.getTree().getDisplay().getSystemColor (SWT.COLOR_INFO_BACKGROUND));

		tbtmAttributes = new TabItem(tabFolder, SWT.NONE);
		tbtmAttributes.setText("Attributes");

		table = new Table(tabFolder, SWT.BORDER | SWT.FULL_SELECTION);
		tbtmAttributes.setControl(table);
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		tblclmnAttributeName = new TableColumn(table, SWT.NONE);
		tblclmnAttributeName.setWidth(141);
		tblclmnAttributeName.setText("attribute name");

		tblclmnAttributeValue = new TableColumn(table, SWT.NONE);
		tblclmnAttributeValue.setWidth(100);
		tblclmnAttributeValue.setText("attribute value");
		
		tbtmUnusedVariables = new TabItem(tabFolder, SWT.NONE);
		tbtmUnusedVariables.setText("Unused variables");
		
		tblUnusedVariables = new Table(tabFolder, SWT.BORDER | SWT.FULL_SELECTION);
		tbtmUnusedVariables.setControl(tblUnusedVariables);
		tblUnusedVariables.setHeaderVisible(true);
		tblUnusedVariables.setLinesVisible(true);
		
		tblclmnVariableName = new TableColumn(tblUnusedVariables, SWT.NONE);
		tblclmnVariableName.setWidth(219);
		tblclmnVariableName.setText("variable name");
		
		tblclmnCount = new TableColumn(tblUnusedVariables, SWT.NONE);
		tblclmnCount.setWidth(123);
		tblclmnCount.setText("count");
		
		tbtmStatistics = new TabItem(tabFolder, SWT.NONE);
		tbtmStatistics.setText("Statistics");
		
		tblStatistics = new Table(tabFolder, SWT.BORDER | SWT.FULL_SELECTION);
		tbtmStatistics.setControl(tblStatistics);
		tblStatistics.setHeaderVisible(true);
		tblStatistics.setLinesVisible(true);
		
		tblclmnData = new TableColumn(tblStatistics, SWT.NONE);
		tblclmnData.setWidth(147);
		tblclmnData.setText("Data");
		
		tblclmnAnalysis = new TableColumn(tblStatistics, SWT.NONE);
		tblclmnAnalysis.setWidth(126);
		tblclmnAnalysis.setText("Analysis");


		// terminate notes thread
		NotesThread.sinitThread();
		try {
			session = NotesFactory.createSession() ;
		} catch (NotesException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	

	public void ssInitScriptSniffing( ) {

		try {
			// initialize lotus notes 
			NotesThread.sinitThread();
			
			// create new fresh session
			session = NotesFactory.createSession() ;

			// create xml document.
			xml_DocFactory = DocumentBuilderFactory.newInstance();
			//xml_DocFactory.setNamespaceAware(true); // never forget this!
			xml_DocBuilder = xml_DocFactory.newDocumentBuilder();
			xml_DOMimpl = xml_DocBuilder.getDOMImplementation();
			xml_Inmemory_Doc = xml_DOMimpl.createDocument(null, "XMLDOCUMENT", null);

			if( str_SelectedServer == null ) {
				sDatabaseTitle =  str_SelectedFile + " (on local)";
			} else {
				sDatabaseTitle = str_SelectedFile  + " (" +  str_SelectedServer + ")";
			}
			lblDatabase.setVisible(true );
			lblDatabase.setText(sDatabaseTitle);

			// add database node
			xml_Database = xml_Inmemory_Doc.createElement("DesignName");
			xml_Database.setAttribute("ss_Name", sDatabaseTitle);
			Element xml_DatabaseNest = null;
			xml_DatabaseNest = xml_Inmemory_Doc.createElement("DesignName");
			xml_DatabaseNest.setAttribute("ss_Name", sDatabaseTitle);
			xml_Database.appendChild(xml_DatabaseNest);
			
			
			
			
			// get chosen database object
			db_Chosendatabase = session.getDatabase( ssMainWindow.str_SelectedServer ,  ssMainWindow.str_SelectedFile);

			

			////////////////////////////////////////////////////
			// Run the big source code job
			///////////////////////////////////////////////////
			final ssProcessAllFunctionsJob jobProcessAll = new ssProcessAllFunctionsJob(db_Chosendatabase, xml_DatabaseNest, xml_Inmemory_Doc);

			
			
			jobProcessAll.addJobChangeListener( new JobChangeAdapter(){
				public void done(IJobChangeEvent event) {
					final IStatus result = event.getResult() ;

					if ( result.isOK()) {
						Display display = Display.getDefault() ;
						display.asyncExec (new Runnable () {
							public void run () {
								try {
									
									ssStartXMLdocPopulateToTree();
																		
								} catch (FileNotFoundException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								} catch (TransformerException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
							}
						}						
						) ;							
					}
				}
			} ) ;
			jobProcessAll.setUser(false);
			jobProcessAll.setPriority(Job.SHORT);
			jobProcessAll.schedule();

			// this thing will show the current job in a dialog box
			PlatformUI.getWorkbench().getProgressService().showInDialog(getSite().getShell(), jobProcessAll);


		} catch (NotesException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}finally {
			   NotesThread.stermThread();
		}
		


	}

	public void ssStartXMLdocPopulateToTree() throws FileNotFoundException, TransformerException{



		// SET INPUT TO OUR XML DOCUMENT
		treeViewer.setInput(xml_Database);
		// SORT THE TREEVIEW
		treeViewer.setSorter( new ViewerSorter());


		
	}

	public void ssGetStatistics(){
		
		
	}
	
	
	public void setFocus() {
		
		
	} 
}

